#ifndef __BYTEREADER_H
#define __BYTEREADER_H

#include <iostream>
#include <stdio.h>
#include <vector>
#include <sstream>

#define PREMATURE_EOF_EXCEPTION 1

// Cache up to 128MB of graph files.
#define MAXIMUM_BUFFER_SIZE 128 * (1024 * 1024)

typedef unsigned char byte;

using namespace std;

namespace simplis
{	
	class ByteReader
	{
	private:
		//! The byte currently considered.
		byte current;

		//! A link to the opened file.
		FILE* file;

		//! How many bits are currently cached?
		int cached;	

		//! The file size.
		unsigned long long fileSize;

		//! The buffer which may contain the whole file.
		byte* buffer;
	
		//! An index for the buffer.
		unsigned int index;

	protected:
		//! Hide the default c'tor.
		ByteReader() { };

		//! Get the next byte from the file.
		void fillCache();

		//! Get a bit.
		int getBit();

	public:
		//! Load a file.
		ByteReader(const string& file);

		//! Retrieve a numeric.
		int readNumeric(int align);
		
		//! Skip the padding.
		void skipPadding();

		//! Have we reached the end of the file?
		bool eof() const;

		//! Rewind the file.
		void rewind();

		//! Close the reader.
		~ByteReader();

	};
}

#endif
