#ifndef EXCEPTION_H_
#define EXCEPTION_H_

// Use custom macros/types.
#include "../include/macros.h"

// Shall we report all exceptions at once?
#define REPORT_IMMEDIATELY

// Use I/O.
#ifdef REPORT_IMMEDIATELY
#include <stdio.h>
#endif

namespace simplis
{
	
	/*!
	 *	This class implements an exception object.
	 *	The exception object is thrown using a macro which 
	 *	states, which part of the application caused the 
	 *	exception. Furthermore the application may append
	 *	information to the exception which will be
	 *	used to describe the occuring error.
	 *
	 *	@author Michael Nett
	 *	@date 02/14/2008
	 */
	class Exception
	{
		
	private:
		//! In which file did the exception occur?
		std::string file;
	
		//! In which line did the exception occur?
		unsigned int line;
	
		//! The appended information.
		std::string data;
		
	protected:
		//! The parameterless constructor is hidden.
		Exception();
	
	public:
		//! The default constructor.
		Exception(const std::string& file, const unsigned int& line, const std::string& data);
		
	};
	
}

#endif /*EXCEPTION_H_*/
