#ifndef GRAPHCOLLECTION_H_
#define GRAPHCOLLECTION_H_

#include "../include/macros.h"
#include "../include/bytebuffer.h"
#include "../include/bytereader.h"
#include "../include/exception.h"
#include "../include/graph.h"
#include "../include/matrix.h"

#include <string>
#include <fstream>

using namespace std;

namespace simplis
{
	class GraphCollection
	{
		
	public:
		typedef int CollectionMode;
		
		const static CollectionMode INPUT_COLLECTION = 1;
		const static CollectionMode OUTPUT_COLLECTION = 2;
	
	private:
		//! The mode of the collection.
		CollectionMode mode;
		
		//! A byte reader in case we read files.
		ByteReader* reader;
		
		//! The output name.
		string outName;
		
	protected:
		//! Default constructor is not visible.
		GraphCollection();
		
	public:
		//! Get the reader.
		ByteReader* getReader() { return reader; };
	
		//! Read from a graph collection.
		GraphCollection(string file, CollectionMode mode);
		
		//! Destroy a graph collection.
		virtual ~GraphCollection();

		//! Write a graph to a file.		
		void write(const Graph& graph, int o2align = 6);

		//! Rewind the graph-file.
		void rewind() { if (reader != 0) reader->rewind(); }

		//! Read a graph from the file.
		Graph* read(int o2align = 6);
	};
}

#endif

