#ifndef MACROS_H_
#define MACROS_H_

// Use the memory control
#include <memory.h>

// Use STL strings.
#include <string>

//! A raw byte (8 bit) type.
typedef unsigned char byte;

//! A raw word (16 bit) type.
typedef unsigned short word;

//! A raw double-word (32 bit) type.
typedef unsigned int dword;

//! A raw quadruple-word (64 bit) type.
typedef unsigned long long qword;

// To avoid stupid errors...
namespace simplis { }

// A hardcoded degree limit.
#define MAX_DEGREE 8

// A hardcoded constant for undefined values.
#define UNDEFINED 0xFFFFFFFFUL

// A bit-grabbing macro.
#define GET_BIT(w,n) ((((w) << (n)) & 0x8000000000000000ULL) != 0)

// Some bit-twiddling macros.
#define SET_BIT(w,n) { w |= (0x0000000000000001ULL << (63 - (n))); }
#define UNSET_BIT(w,n) { w &= ~(0x0000000000000001ULL << (63 - (n))); }

/*
 *	A macro to throw exceptions.
 */
#define exception(data) Exception(__FILE__, __LINE__, (data))

// Some matrix macros.
#define MATRIX_BIT_NEED(n) (((n)*(n))-(n))
#define PAD_BITS_TO_QWORD(n) (((n)/64)+((n)%64==0?0:1))
#define MATRIX_BITMAP_TRANSFORM(i,j,n) (((2*(n)-1)*(i)-(i)*(i))/2+(j)-(i)-1)

/*
 *	This macro is used to verify any assertions placed
 *	in the programm code. Be aware, that the expressions
 *	used in the assertions will be evaluated in that
 *	process.
 */
//#define COMPILE_ASSERTIONS
#ifdef COMPILE_ASSERTIONS
#define assert(phi) { if(!(phi)) { std::string ___exc_descr = "Assertion \""; ___exc_descr += #phi; ___exc_descr += "\" failed!"; throw exception(___exc_descr); } }
#else
#define assert(phi) {}
#endif

#endif /*MACROS_H_*/
