#include "../include/berko.h"

simplis::BerkowitzHash::BerkowitzHash(int i) {
	this->cap = i;
}

unsigned long simplis::BerkowitzHash::_hash(unsigned long X, int m) {
	unsigned long long x = static_cast<unsigned long long>(X);
	unsigned long long T = 2654435769ULL * x & 0x00000000FFFFFFFFULL;
	return T >> (32 - m);
}

string simplis::BerkowitzHash::applyHash(simplis::Matrix* M)
{
	// How big is the matrix?
	unsigned int size = M->size();

	// We need to convert the matrix into an integer matrix.
	int** a = new int*[size];
	for (unsigned int k = 0; k < size; ++k) a[k] = new int[size];
	for (int i = 0; i < static_cast<int>(size); ++i) 
	{
		for(int j = 0; j < static_cast<int>(size); ++j)
		{
			a[i][j] = M->get(i,j) ? 1 : 0;
		}
	}

	// Perform Berkowitz algorithm to determine the coefficients of the char. polynomial.
	int* p = recursiveHash(a, 0, size);
	stringstream hash;
	int key = 0;
	for (unsigned k = 0; k < size; ++k) {
		int u = p[size - 1 - k];
		key ^= _hash(u, DEFAULT_CAP_EXP);
	}
	hash << key;
	for (unsigned k = 0; k < size - 1; ++k) delete [] a[k];
	delete [] a;
	delete [] p;
	return hash.str();
}

int* simplis::BerkowitzHash::recursiveHash(int** A, const unsigned int& step, const unsigned int& size)
{
	// How large is the considered fraction of the matrix?
	int width = size - step;

	// The coefficients of the current step.
	int* p = new int[width + 1];

	// Check for the trivial base-case.
	if (width == 1)
	{
		p[0] = 1;
		p[1] = -A[0][0];
		return p;
	}

	// Allocate space for considered matrices and vectors.
	int** M = new int*[width - 1];
	int** Mi = new int*[width - 1];
	for (int k = 0; k < width - 1; ++k)
	{
		M[k] = new int[width - 1];
		Mi[k] = new int[width - 1];
		memset(M[k], 0, sizeof(int) * (width - 1));
		memset(Mi[k], 0, sizeof(int) * (width - 1));
	}
	int* R = new int[width - 1];
	int* S = new int[width - 1];

	// Copy the submatrices M,R,S.
	for (int i = 0; i < width - 1; ++i) 
	{
		R[i] = A[0][i+1];
		S[i] = A[i+1][0];
		for (int j = 0; j < width - 1; ++j) M[i][j] = Mi[i][j] = A[i+1][j+1];
	}

	// Determine the i-2nd power of M.
	for (int k = 0; k < width + 1; ++k)
	{
		int a = 0;
		// We can calculate some powers easier than others.
		switch (k)
		{
		case 0:
			a = -1;
			break;
		case 1:
			a = A[0][0];
			break;
		case 2:
			for (int l = 0; l < width - 1; ++l) a += R[l] * S[l];
			break;
		default:
			for (int i = 0; i < width - 1; ++i)
			{
				int t = 0;
				for (int j = 0; j < width - 1; ++j) t += R[j] * Mi[i][j];
				a += t * S[i];
			}
			int* T = new int[width - 1];
			for (int i = 0; i < width - 1; ++i)
			{
				memset(T, 0, sizeof(int) * (width - 1));
				for (int j = 0; j < width - 1; ++j)
				{
					for (int n = 0; n < width - 1; ++n)
					{
						T[j] += Mi[n][i] * M[j][n];
					}
				}
				for (int n = 0; n < width - 1; ++n) Mi[n][i] = T[n];
			}
			delete [] T;
			T = 0;
		}

		// We've got our k-th coefficient (This is total voodoo!)
		p[k] = -a;
	}

	// Start the next recursive step.
	int* pN = recursiveHash(M, step + 1, size);

	// Combine the results.
	int* P = new int[width + 1];
	memset(P, 0, sizeof(int) * (width + 1));
	for (int k = 0; k < width + 1; ++k)
	{
		for (int l = 0; l < width; ++l)
		{
			int a = 0;
			if (k >= l)
			{
				a = p[k-l];
			}
			P[k] += a * pN[l];
		}
	}

	// Cleanup.
	delete [] p;
	delete [] pN;
	delete [] R;
	delete [] S;
	for (int k = 0; k < width - 1; ++k) 
	{
		delete [] M[k];
		delete [] Mi[k];
	}
	delete [] M;
	delete [] Mi;

	return P;
}

