#include "../include/bytebuffer.h"

simplis::ByteBuffer::ByteBuffer()
{
	bytes.clear();
	current = 0x00;
	position = 0;
}

/*!
 *	Write a number with a certain bit-alignment to the buffer. According
 *	to the specification numbers are stored using the least-significant
 *	bit first.
 */
void simplis::ByteBuffer::writeNumeric(int value, int alignment)
{
	do
	{
		// Write the least significant bit from value.
		putBit((value & 1) > 0);
		
		// Shift the value.
		value >>= 1;
		
		// Decrease the residual alignment.
		--alignment;
	}
	while (alignment > 0);
}

/*!
 *	Convert the buffer to a string.
 */
string simplis::ByteBuffer::toString() const
{
	ostringstream s;
	for (vector<byte>::const_iterator b = bytes.begin(); b != bytes.end(); ++b)
	{
		for(int k = 7; k >= 0; --k)
		{
			int digit = ((*b) >> k) & 1;
			s << digit;
		}
	}	
	return s.str();
}

/*!
 *	This method finishes the bytebuffer, by writing the
 *	content of the current byte into the buffer. The
 *	data is padded to the next byte by filling it up with
 *	zeroes.
 */
void simplis::ByteBuffer::flush()
{
	if (position > 0)
	{
		while (!putBit(false)) { }
	}
}	

/*!
 *	Writes a bit to the buffer. Returns wether the buffer was
 *	flushed or not.
 */
bool simplis::ByteBuffer::putBit(bool bit)
{
	// Create a mask that consists of 0s and has 'bit' as the first value.
	byte mask = bit ? 0x80 : 0x00;

	// Shift the mask to the right until the bit is at the desired position.
	mask >>= position;

	// Add the shifted mask to the buffer.
	current |= mask;

	// Increment the position.
	++position;

	// If the byte is full, we print it.
	if (position == 8)
	{
		position = 0;
		bytes.push_back(current);
		current = 0x00;
		return true;
	}	
	return false;
}

/*!
 *	Ready the buffer for writing.
 */
void simplis::ByteBuffer::pack()
{
	// Flush the buffer.
	flush();
}

/*!
 *	Appends the buffer to a file.
 */
#include <iostream>
using namespace std;
void simplis::ByteBuffer::appendToFile(const string& name)
{
	// Prepare the buffer for writing.
	pack();

	// Try to open the file.
	FILE* output = fopen(name.c_str(), "a+b");
	if (output == 0)
	{
		cerr << "Failed to open the file." << endl;
		return;
	}

	// Convert the vector of the buffer into a byte-array.
	byte* data = new byte[bytes.size()];

	// Copy the buffer.
	for (unsigned int i = 0; i < bytes.size(); ++i) 
	{
		data[i] = bytes[i];
	}

	// Write the data to the file.
	if (fwrite(data, bytes.size(), 1, output) != 1)
	{
		cerr << "Failed to write to the output file." << endl;
	}

	// Close the file.
	fclose(output);

	// Release the byte array.
	delete [] data;
}
