#include "../include/bytereader.h"

#include <iostream>
using namespace std;

simplis::ByteReader::~ByteReader()
{
	// Close any files.
	if (file) fclose(file);

	// Release the buffers.
	if (buffer) 
	{
		delete [] buffer;
		buffer = 0;
	}
}

simplis::ByteReader::ByteReader(const std::string& filename)
{
	// Try to open the file.
	file = fopen(filename.c_str(), "rb");

	// Try to determine the length of the file.
	if (file)
	{
		fseek(file, 0, SEEK_END);
		fileSize = (unsigned long long)(ftell(file));
		fseek(file, 0, SEEK_SET);
	}
	else
	{
		fileSize = MAXIMUM_BUFFER_SIZE + 1; // Provokes unbuffered mode.
	}

	// Determine wether we buffer the whole file into memory.
	if (fileSize > MAXIMUM_BUFFER_SIZE) 
	{
		buffer = 0;
		index = 0;
	}
	else
	{
		// Allocate the buffer.
		buffer = new byte[fileSize];

		// Read the file into the buffer in one sweep.
		if (fread(this->buffer, fileSize, 1, this->file) != 1)
		{
			cerr << "Failed to buffer the file!" << endl;
		}

		// Set the index to point to the beginning of the file.
		index = 0;

		// Close the link to the file
		fclose(file);
		file = 0;
	}

	// We have nothing in the cache.
	cached = 0;
	current = 0x00;
}

void simplis::ByteReader::skipPadding()
{
	cached = 0;
}

bool simplis::ByteReader::eof() const
{
	// Are we using the buffered mode?
	if (buffer)
	{
		return cached == 0 && index >= fileSize;
	}
	else
	{
		return feof(file) && cached == 0;
	}
}

void simplis::ByteReader::fillCache()
{
	// Are we in file mode?
	if (buffer == 0)
	{
		// Load a byte from the file.
		if (fread(&current, 1, 1, file) != 1) throw PREMATURE_EOF_EXCEPTION;
		cached = 8;
	} 
	else
	{
		if (index >= fileSize) throw PREMATURE_EOF_EXCEPTION;
		current = buffer[index++];
		cached = 8;
	}
}

int simplis::ByteReader::getBit()
{
	// Something in the cache?
	if (cached == 0) fillCache();

	// Get the first bit of the cached byte.
	int bit = current & 0x80;

	// Remove the first element from the cache.
	current <<= 1;
	--cached;

	// Return the extracted bit.
	return bit;
}

int simplis::ByteReader::readNumeric(int align)
{
	// Store the result here.
	int extracted = 0;

	// Read bits, until the alignment is reached.
	for (int k = 0; k < align; ++k)
	{
		// Get the next bit.
		int bit = getBit() == 0 ? 0 : 1;

		// Shift that bit to the desired position.
		bit <<= k;

		// Mask the bit onto the result.
		extracted |= bit;
		

	}
	return extracted;
}

void simplis::ByteReader::rewind()
{
	// Flush the cache.
	cached = 0;
	current = 0x00;

	// Are we in buffered mode?
	if (buffer)
	{
		// Rewind the buffer's index.
		index = 0;
	}
	else
	{
		// Rewind the file.
		fseek(file, 0, SEEK_SET);
	}
}
