
/*
 *	EDGE-Tool
 *
 *	This tool is part of the simplis toolkit. In this tool
 *	the edges on the outer orbit are filled in. Therefore
 *	we can limit the intra-orbital edges by considering
 *	a minimum and maximum degree that no vertex must
 *	exceed.
 */

// External dependencies.
#include <stddef.h>
#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <string>
#include <sstream>
#include <iostream>
#include <stdio.h>
#include <list>
#include <map>
#include <time.h>
/*
#include <fstream>
#include "../common/include/graph.h"
#include "../common/include/graphcollection.h"
#include "../common/include/bytebuffer.h"
*/

// A new type.
typedef unsigned char byte;

// Default namespaces.
using namespace std;
//using namespace simplis;

// Global variables.
string inputPath = "";
string outputPath = "";
time_t startTime = static_cast<time_t>(0);

// Prototypes.
bool parseArguments(int, char**);
bool is_dir(const string& path);
void printSyntax();

// Point of entry.
int main(int argc, char** argv)
{
	// Try to parse the arguments.
	if (!parseArguments(argc, argv))
	{
		printSyntax();
		return 0;
	}
	
	// Print an initial message.
	cout << "SMERGE merging sets..." << endl;
	startTime = time(0);

	// Try to open the source directory.
	DIR* dir = opendir(inputPath.c_str());
	if (dir == 0)
	{
		cerr << "SMERGE could not open the directory " << inputPath.c_str() << endl;
		return 0;
	}

	// Create a list of input files.
	list<string> inputFiles;

	// Get all the files from the directory.
	struct dirent* file;
	while ((file = readdir(dir)) != 0)
	{
		// Create the full path.
		string path = inputPath;
		if (inputPath[inputPath.size() - 1] != '/') path += '/';
		path += file->d_name;

		// Is the entity a directory?
		if (!is_dir(path))
		{
			inputFiles.push_back(path);
		}
	}
	
	// We're done with the directories.
	closedir(dir);

	/*
	// Create a graph collection to store the graphs.
	GraphCollection collection(outputPath, GraphCollection::OUTPUT_COLLECTION);
	*/
	// Open the output file.
	FILE* out = fopen(outputPath.c_str(), "wb");
	
	// Iterate through all the available input files.
	for (list<string>::iterator in = inputFiles.begin(); in != inputFiles.end(); ++in)
	{
		// Determine the filename.
		string filename = *in;

		/*
		// Open the graph file.
		GraphCollection* input = new GraphCollection(filename.c_str(), GraphCollection::INPUT_COLLECTION);
		try
		{
			while (true)
			{
				// Get the next graph from the source file.
				Graph* next = input->read();

				// Store this graph in the output collection.
				collection.write(*next);

				// Delete the graph.
				delete next;
			}
		}
		catch (int interrupt)
		{
		}		

		// Release the input file.
		delete input;
		*/

		// Open the input file.
		FILE* in = fopen(filename.c_str(), "rb");

		// How big is the file?
		fseek(in, 0, SEEK_END);
		unsigned long long filesize = ftell(in);
		fseek(in, 0, SEEK_SET);

		// Prepare a buffer for the file.
		byte* buffer = new byte[filesize];

		// Copy the file.
		fread(buffer, filesize, 1, in);
		fwrite(buffer, filesize, 1, out);
	
		// Clean-up
		delete [] buffer;
		fclose(in);
	}

	// Print a result message.
	time_t duration = (time(0) - startTime);
	cout << "SMERGE finished in " << duration << " seconds." << endl << endl;
	return 0;	
}

void printSyntax()
{
	cerr 	<< "Invocation: smerge [OPTIONS]" << endl 
			<< "Folds the orbital vertices of a set of graphs." << endl
			<< endl
			<< "Arguments mandatory for long options are also mandatory for short options." << endl
			<< "  -i, --input     The input directory." << endl
			<< "  -o, --output    A file to store the merged graphs." << endl;
}

bool parseArguments(int argc, char** argv)
{
	// We need at least two arguments (plus the first argument that is an OS-default).
	if (argc != 3) 
	{
		return false;
	}
	
	// In case we have enough arguments, we parse every one of them.
	for (int k = 1; k < argc; ++k)
	{
		// Convert the argument into a key and a value.
		string text = argv[k];
		size_t separatorPos = text.find_first_of('=');
		string key = text.substr(1, separatorPos - 1);
		string valueText = text.substr(separatorPos + 1);
		istringstream value(valueText);
		
		// Check the key and extract the value accordingly.
		if (key == "i" || key == "-input")
		{
			value >> inputPath;
		}
		else if (key == "o" || key == "-output")
		{
			value >> outputPath;
		}
	}
	
	// Any vital parameter not given?
	if (inputPath == "" || outputPath == "")
	{
		return false;
	}
	return true;
}

bool is_dir(const string& path)
{
	DIR* dir = opendir(path.c_str());
	if (dir)
	{
		closedir(dir);
		return true;
	}
	else
	{
		return false;
	}
}
